/*

Modular HTML Version 2
By Joshua Moss (Bandock)

This library acts as an interface for HTML and Javascript access.
Knowledge of HTML and/or Javascript is required to use this library
efficiently.  You may use any reference for HTML tags and Javascript
functions.

Using Javascript can either be done by using /jsfunc and /jsobj
objects with the MH_JAVASCRIPT enabled inside the environment file.
However, it is also doable by hand which does not require
MH_JAVASCRIPT to be defined.


Defines:
	MH_JAVASCRIPT - This define is used to enable the /jsfunc and
	/jsobj objects.  If you're going to define it, make sure to define
	it inside the environment file of your project to prevent
	compilation issues.


Constants:
	JS_PREFIX - MH_JAVASCRIPT must be defined in the environment file
	to use this constant.  It makes it easier to use prefix operators
	for the AdjustVar procedure as a flag..

	JS_POSTFIX - MH_JAVASCRIPT must be defined in the environment file
	to use this constant.  It makes it easier to use postfix operators
	for the AdjustVar procedure as a flag.

	JS_DOWHILESTART - MH_JAVASCRIPT must be defined in the environment
	file to use this constant.  It is used to set the starting point
	for a do while statement with the DoWhile procedure.

	JS_DOWHILEEND - MH_JAVASCRIPT must be defined in the environment
	file to use this constant.  It is used to set the ending point for
	a do while statement with the DoWhile procedure.


/htmlobj (HTML Object)
----------------------
Inherits from the /datum type.

Variables:
	headdata - Used to store head data within the head tag.

	bodydata - Used to store body data within the body tag.

	scriptdata - Used to store script data inside the head tag.
	Javascript is currently the scripting language this library
	will use.

	bodyonload - Used to store the onload attribute of the body tag.


Procedures:
	New(onload)
		Purpose:  To create the html object that can be altered
		through the use of different procedures.  It can be
		overridden, just as long it makes a call to the parent (for
		setting the onload function).

		onload - Used to set the onload attribute for the body tag.
		Useful for calling Javascript functions either user-defined
		or built-in.

	Topic(href,href_list[])
		Purpose:  A customized Topic that enables sending clients
		to the html object created page.  Make sure to call the parent
		when overriding this procedure to properly setup link data.

		href - Left there for compatibility with the original Topic
		procedure.

		href_list[] - Uses this inherited argument to allow access
		to the html object's page.  Much like the original Topic,
		it can consist of other topic data for dynamic pages.

	AddHead(data)
		Purpose:  To add any data inside the head tag.  Useful for
		some cases.

		data - Data to add on to the headdata variable.

	AddBody(data)
		Purpose:  To add any data inside the body tag.  That is where
		most HTML data go normally.

		data - Data to add on to the bodydata variable.

	AddHtmlObjLink(htmlobj/link,client/cusr,topicdata,info)
		Purpose:  To add a working link to a page created from an
		existing html object.  This enables creating multiple pages
		inside a single DM program.

		htmlobj/link - Specifies the HTML object to link to.  Must
		have a working reference or it may not function at all.

		client/cusr - Specifies the user the page will be sent to
		upon clicking the link.

		topicdata - Provided as a way to access pages through topic
		data.  This enables customizable Topic procedures for
		inherited objects.  It is a completely optional argument.

		info - Sets up the info shown on the page.

	AddHtmlObjLinkInput(htmlobj/link,client/cusr)
		Purpose:  To add two input elements containing the necessary
		data for form-based actions.  This procedure is best used
		inside a working form.

		htmlobj/link - Same functionality as the AddHtmlObjLink
		procedure.

		client/cusr - Same functionality as the AddHtmlObjLink
		procedure.

	GetHtmlObjLink(htmlobj/link,client/cusr,topicdata)
		Purpose:  To return an html object link in pure link text
		form.  Handy for calls made through Javascript to access
		pages.

		htmlobj/link - Same functionality as the AddHtmlObjLink
		procedure.

		client/cusr - Same functionality as the AddHtmlObjLink
		procedure.

		topicdata - Same functionality as the AddHtmlObjLink
		procedure.

	StartForm(id,action,method)
		Purpose:  To create the beginnings of a form, which is handy
		for form creation.  It can also be done by hand if one
		desires.

		id - To specify id data, which is handy for Javascript access.
		It is optional unless needed for communication.

		action - To specify the link when submitting data.  Make
		sure to provide a way to submit data if this argument is
		ever used.  It is optional for cases that do not require
		action.

		method - To specify method either as "get" or "post".

	EndForm()
		Purpose:  To create the endings of a form.  Also can be done
		by hand if one desires.

	StartScript()
		Purpose:  To add the opening script tag specifying Javascript
		functionality.

	AddScript(data)
		Purpose:  To add scripting data inside the script tag.
		Before calling this procedure, make sure you make a call to
		the StartScript procedure.

		data - Data to add on to the scriptdata variable.  If
		MH_JAVASCRIPT is defined in the environment file, you may
		use the GetJSFunc procedure in both /jsfunc and /jsobj types
		to add scripting data.

	EndScript()
		Purpose:  To add the closing script tag to the script.  Make
		sure to call this after adding necessary script data.

	GetHtml()
		Purpose:  To return every bit of HTML-related data that
		includes headdata, scriptdata (inside the head tag), bodydata,
		and bodyonload.  This is how one can get HTML data for later
		use.


/jsfunc (Javascript Function)
-----------------------------
Inherits from the /datum type.  MH_JAVASCRIPT must be defined to use.

Variables:
	jsfuncname - Used to store the function name.

	jsfuncdata - Used to store the actual function script data
	for later insertion.

	list/jsfargs - Used for setting argument names for the function.

	list/jsfvars - Used for setting most variables for the function,
	usually at the start.


Procedures:
	New(jsfuncname)
		Purpose:  To create the javascript function object that can
		be altered using provided procedures.  Any variables that
		are initialized in jsfvars will be stored in the function
		upon creation.  Objects that inherit from this type should
		call the parent with the function name argument for proper
		setup.

		jsfuncname - Used for setting up the function name by storing
		it into the object's jsfuncname variable.

	AddVar(varname,data,...)
		Purpose:  To initialize a new variable inside the javascript
		function object provided with a variable name and data.
		Normally called automatically through the New procedure
		utilizing the jsfvars list.  However, you may call it manually
		if you feel the need to.

		varname - Used for specifying the name of the new variable.

		data - Used for specifying the data of the new variable.
		If the data is a type of string, you can either just type in
		a normal string or utilize the '*' operator as the first
		character.  Enables direct-access mode to grant more control
		such as setting to other variables and more.  If the data is
		a type of number, it will store it as such.  If the data is
		a /jsobj type, it enables object creation.  If the data is a
		/jsfunc type, it points to the function (like a function
		pointer).

		... - Used for storing different arguments for object
		creation.

	SetVar(varname,operator,data,...)
		Purpose:  To set existing variables inside the javascript
		function object using an existing variable name and specify
		data.

		varname - Used for specifying the name of the existing
		variable.

		operator - Used for specifying the operator to specify before
		the equal sign.  This argument can be set to null if no
		operator is required. Currently non-functional for object
		creation and function pointers.

		data - Used for specifying the data of the existing variable.
		Functions similarly to the AddVar procedure.

		... - Similar function to the AddVar procedure.

	AdjustVar(varname,operator,flag)
		Purpose:  To add prefix/postfix operators next to the
		varname to alter data.

		varname - Used for specifying the name of the existing
		variable.

		operator - Operator to use for prefix/postfix operations.
		Make sure to use proper operators when dealing with procedure.

		flag - Used to specify JS_PREFIX or JS_POSTFIX for type of
		operation.

	AddCall(funcname,...)
		Purpose:  To add a function call to an existing function
		specified by funcname and provide any arguments.

		funcname - Used to specify what function to call.

		... - Used for adding arguments to a function call.  The '*'
		operator can be used to enable direct access to variables as
		well as provide more control.  It is even possible to call
		another function (if it returns anything).

	If(condition)
		Purpose:  To implement an if statement provided with a
		condition.  Code can then be added through the use of other
		procedures until a call to EndCondition is made.

		condition - Used to specify the condition it must meet to
		pass.  Must provide the condition in string form for it to
		function properly (though it is possible to use numbers in
		some cases without a need for the string).

	ElseIf(condition)
		Purpose:  To implement an elseif statement provided with a
		condition.  Before calling this procedure, you must make a
		call to the If procedure and end it with the EndCondition
		procedure.  After that, it is used in the same way as the
		If procedure and ended with the EndCondition procedure.

		condition - Works like the one in the If procedure.

	Else()
		Purpose:  To implement an else statement with no condition
		to provide.  Before calling this procedure, you must make a
		call to the If procedure and end it with the EndCondition
		procedure.  After that, it is used in the same way as the
		If procedure and ended with the EndCondition procedure.

	While(condition)
		Purpose:  To implement a while statement provided with a
		condition.  Code can then be added through the use of other
		procedures until a call to EndCondition is made.

		condition - Works like the one in the If procedure.

	DoWhile(flag,condition)
		Purpose:  To implement a dowhile statement provided with
		the proper flag and condition.  Must be called once with the
		JS_DOWHILESTART flag to start the dowhile statement.  Then
		add any code through the use of other procedures until ready
		to make another call to this procedure again with the
		JS_DOWHILEEND flag.  There is no need to call the EndCondition
		procedure.

		flag - Flag to use with this procedure.  JS_DOWHILESTART and
		JS_DOWHILEEND are available for use with this procedure.

		condition - Works like the one in the If, but only functions
		with the JS_DOWHILEEND flag.

	For(ival,condition,operation)
		Purpose:  To implement a for statement provided with a
		condition.  Initial value and operation arguments are optional
		in case you have a special use for this procedure.  You must
		call the EndCondition procedure to end it.

		ival - Specifies initial value either for a new temporary
		variable or an existing variable.  It is not required and
		can be left null.

		condition - Works like the one in the If procedure.  Only
		argument required to function.

		operation - Used for setting up operators inside the for
		statement.  It is not required and can be left null.

	EndCondition()
		Purpose:  To end the If, ElseIf, Else, While, or For
		procedures.  It is possible to call this several times in
		certain nested conditional statements.

	GetJSFunc()
		Purpose:  To return the entire function object's script data
		for adding on to an existing script using the /htmlobj's
		AddScript procedure.


/jsobj (Javascript Object)
--------------------------
Inherits from the /jsfunc type.  MH_JAVASCRIPT must be defined to use.

Variables:
	list/jsovars - Used to initialize/setup most object variables
	through the constructor.


Procedures:
	New()
		Purpose:  To create the javascript object that can be altered
		using provided procedures.  It makes a call to the parent
		procedure before setting up object variables specified in
		the jsovars list.  As mentioned from the /jsfunc type, you
		should call the parent for any object that inherits from this
		type for proper setup.

	SetObjVar(varname,operator,data,...)
		Purpose:  To create new object variables or set existing ones.
		Other than that, it functions very similar to both AddVar
		and SetVar procedures of the /jsfunc type.

		varname - Works like the one found in AddVar and SetVar
		procedures.

		operator - Works like the one found in the SetVar procedure.

		data - Works like the one found in the AddVar and SetVar
		procedures.

		... - Works like the one found in the AddVar and SetVar
		procedures.

	AdjustObjVar(varname,operator,flag)
		Purpose:  To add prefix/postfix operators to the object's
		varname to alter data.  It functions very similar to /jsfunc's
		AdjustVar procedure.

		varname - Works like the one found in the AdjustVar procedure.

		operator - Works like the one found in the AdjustVar
		procedure.

		flag - Works like the one found in the AdjustVar procedure.

*/